==> lm-toptips-0.1.2.tgz: package/src/MarqueeX.js <==
/**
 * Created by liuduan<liuduan.05.05@163.com>.
 * ComponentName MarqueeX
 * Desc 跑马灯显示文字-水平方向
 * GroupName lm-component
 */
import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';

import classNames from 'classnames';
import './MarqueeX.scss';

export default class MarqueeX extends Component {

    static propTypes = {
        prefixCls: PropTypes.string,
        className: PropTypes.string,
        type: PropTypes.string,
        speed: PropTypes.number,
        startDelay: PropTypes.number,
        endDelay: PropTypes.number,
        loop: PropTypes.bool
    };

    static defaultProps = {
        prefixCls: 'lm',
        className: '',
        /* ===== 计时器相关属性 ===== */
        type: 'marquee',    // 显示模式：marquee-跑马灯 | ellipsis-显示... | clip-修剪文本 | break-折行
        speed: 20,          // 动画移动距离/每秒
        startDelay: 800,    // 动画单次播放开始延时时间
        endDelay: 500,      // 动画单次播放结束停留时间
        loop: true          // 是否循环播放
    };

    /* ======= 实例属性 ======= */
    state = {
        overflowWidth: 0,  // 文字超出wrap部分宽度
        animatedWidth: 0   // 文字移动的距离
    };

    _marqueeTimer; // 计时器函数

    componentDidMount() {
        this._measureText(this._startAnimation);
    }

    componentWillReceiveProps(nextProps) {
        if ('children' in nextProps) {
            this.setState({
                overflowWidth: 0,
                animatedWidth: 0
            }, () => {
                if (this._marqueeTimer) this._marqueeTimer = clearTimeout(this._marqueeTimer);
                this._measureText(this._startAnimation);
            });
        }
    }

    componentWillUnmount() {
        clearTimeout(this._marqueeTimer);
    }

    render() {

        const { children, className, style, prefixCls, type, speed, startDelay, endDelay, loop, ...others } = this.props;
        const { overflowWidth, animatedWidth } = this.state;

        const cls = classNames('lm-marqueeX', `${prefixCls}-marqueeX`, className);
        const textCls = classNames('lm-marqueeX-text', `${prefixCls}-marqueeX-text`);

        let  marqueeStyle= {}, ellipsisStyle = {}, breakStyle = {}, wrapStyle = {}, textStyle = {}, renderChildren;
        if (type === 'marquee') {
            marqueeStyle = {
                textAlign: overflowWidth > 0 ? 'left' : 'center',
                whiteSpace: 'nowrap',
                overflowX: 'hidden',
                height: 20,
                display: 'inline-block',
                position: 'relative',
                right: animatedWidth,
            }
        }

        if (type === 'ellipsis' || type === 'clip') {
            ellipsisStyle = {
                textAlign: 'center',
                whiteSpace: 'nowrap',
                textOverflow: type,
                overflowX: 'hidden',
                height: 20,
                display: 'block'
            }
        }

        if (type === 'break') {
            breakStyle = {
                wordWrap: 'break-word',
                wordBreak: 'normal',
                whiteSpace: 'normal',
                height: 'auto',
                display: 'block'
            };
        }

        wrapStyle = {
            ...style,
            height: type === 'break' ? 'auto' : 20
        }

        textStyle = {
           ...marqueeStyle,
           ...ellipsisStyle,
           ...breakStyle
        };

        if (type !== "marquee" && typeof children === 'object' && children.type) {
            renderChildren = React.cloneElement(
                children,
                {style: textStyle}
            );
        } else {
            renderChildren = children;
        }
        return (
            <div className={cls} style={wrapStyle} {...others}>
                <div ref={el => this.textRef = el} className={textCls} style={textStyle}>
                    {renderChildren}
                </div>
            </div>
        )
    }

    /* 私有方法：计算文字超出marquee容器范围的宽度 */
    _measureText(cb) {
        const wrap = ReactDOM.findDOMNode(this);
        const text = this.textRef;
        if (wrap && text) {
            // 获取marquee容器宽度
            const wrapWidth = wrap.offsetWidth;
            // 获取文字宽度
            const textWidth = text.offsetWidth;
            // 获取文字超出容器部分宽度
            const overflowWidth = textWidth - wrapWidth;
            this.setState({
                overflowWidth
            }, cb.bind(this));
        }
    }

    /* 私有方法：计时器-滚动计算 */
    _startAnimation() {
        if (this._marqueeTimer) {
            clearTimeout(this._marqueeTimer);
        }
        const { type, speed, startDelay, endDelay, loop } = this.props;
        const timeout = 1 / speed * 1000;

        const animate = () => {
            let { overflowWidth, animatedWidth } = this.state;
            animatedWidth++;
            const isRoundOver = animatedWidth > overflowWidth;
            // 判断单次播放完毕，若播放完毕再判断是否需要循环播放
            if (isRoundOver) {
                if (loop) {
                    // 开启循环播放函数
                    animatedWidth = 0;
                    this._marqueeTimer = setTimeout(() => {
                        this.setState({
                            animatedWidth,
                        }, () => {
                            this._marqueeTimer = setTimeout(animate, startDelay);
                        });
                    }, endDelay);
                } else {
                    return;
                }
            } else {
                // 播放过程动画函数
                this.setState({
                    animatedWidth,
                }, () => {
                    this._marqueeTimer = setTimeout(animate, timeout);
                });
            }
        };

        // 如果文字内容超出wrap, 开启首次滚动动画
        if (this.state.overflowWidth > 0 && type === 'marquee') {
            this._marqueeTimer = setTimeout(animate, startDelay);
        }
    }

}

==> lm-toptips-0.1.3.tgz: package/src/MarqueeX.js <==
/**
 * Created by liuduan<liuduan.05.05@163.com>.
 * ComponentName MarqueeX
 * Desc 跑马灯显示文字-水平方向
 * GroupName lm-component
 */
import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';

import classNames from 'classnames';
import './MarqueeX.scss';

export default class MarqueeX extends Component {

    static propTypes = {
        prefixCls: PropTypes.string,
        className: PropTypes.string,
        type: PropTypes.string,
        speed: PropTypes.number,
        startDelay: PropTypes.number,
        endDelay: PropTypes.number,
        loop: PropTypes.bool
    };

    static defaultProps = {
        prefixCls: 'lm',
        className: '',
        /* ===== 计时器相关属性 ===== */
        type: 'marquee',    // 显示模式：marquee-跑马灯 | ellipsis-显示... | clip-修剪文本 | break-折行
        speed: 20,          // 动画移动距离/每秒
        startDelay: 800,    // 动画单次播放开始延时时间
        endDelay: 500,      // 动画单次播放结束停留时间
        loop: true          // 是否循环播放
    };

    /* ======= 实例属性 ======= */
    state = {
        overflowWidth: 0,  // 文字超出wrap部分宽度
        animatedWidth: 0   // 文字移动的距离
    };

    _marqueeTimer; // 计时器函数

    componentDidMount() {
        this._measureText(this._startAnimation);
    }

    componentWillReceiveProps(nextProps) {
        if ('children' in nextProps) {
            this.setState({
                overflowWidth: 0,
                animatedWidth: 0
            }, () => {
                if (this._marqueeTimer) this._marqueeTimer = clearTimeout(this._marqueeTimer);
                this._measureText(this._startAnimation);
            });
        }
    }

    componentWillUnmount() {
        if (this._marqueeTimer) this._marqueeTimer = clearTimeout(this._marqueeTimer);
    }

    render() {

        const { children, className, style, prefixCls, type, speed, startDelay, endDelay, loop, ...others } = this.props;
        const { overflowWidth, animatedWidth } = this.state;

        const cls = classNames('lm-marqueeX', `${prefixCls}-marqueeX`, className);
        const textCls = classNames(`${prefixCls}-marqueeX-text`, {
            [`lm-marqueeX-text-${type}`]: type
        });

        let wrapStyle, textStyle = {}, renderChildren;

        wrapStyle = {
            height: type === 'break' ? 'auto' : 20,
            ...style
        }

        if (type === 'marquee') {
            textStyle = {
                textAlign: overflowWidth > 0 ? 'left' : 'center',
                right: animatedWidth
            }
        }

        if (type !== "marquee" && typeof children === 'object' && children.type) {
            renderChildren = React.cloneElement(
                children,
                {className: textCls}
            );
        } else {
            renderChildren = children;
        }
        
        return (
            <div className={cls} style={wrapStyle} {...others}>
                <div ref={el => this.textRef = el} className={textCls} style={textStyle}>
                    {renderChildren}
                </div>
            </div>
        )
    }

    /* 私有方法：计算文字超出marquee容器范围的宽度 */
    _measureText(cb) {
        const wrap = ReactDOM.findDOMNode(this);
        const text = this.textRef;
        if (wrap && text) {
            // 获取marquee容器宽度
            const wrapWidth = wrap.offsetWidth;
            // 获取文字宽度
            const textWidth = text.offsetWidth;
            // 获取文字超出容器部分宽度
            const overflowWidth = textWidth - wrapWidth;
            this.setState({
                overflowWidth
            }, cb.bind(this));
        }
    }

    /* 私有方法：计时器-滚动计算 */
    _startAnimation() {
        if (this._marqueeTimer) {
            clearTimeout(this._marqueeTimer);
        }
        const { type, speed, startDelay, endDelay, loop } = this.props;
        const timeout = 1 / speed * 1000;

        const animate = () => {
            let { overflowWidth, animatedWidth } = this.state;
            animatedWidth++;
            const isRoundOver = animatedWidth > overflowWidth;
            // 判断单次播放完毕，若播放完毕再判断是否需要循环播放
            if (isRoundOver) {
                if (loop) {
                    // 开启循环播放函数
                    animatedWidth = 0;
                    this._marqueeTimer = setTimeout(() => {
                        this.setState({
                            animatedWidth,
                        }, () => {
                            this._marqueeTimer = setTimeout(animate, startDelay);
                        });
                    }, endDelay);
                } else {
                    return;
                }
            } else {
                // 播放过程动画函数
                this.setState({
                    animatedWidth,
                }, () => {
                    this._marqueeTimer = setTimeout(animate, timeout);
                });
            }
        };

        // 如果文字内容超出wrap, 开启首次滚动动画
        if (this.state.overflowWidth > 0 && type === 'marquee') {
            this._marqueeTimer = setTimeout(animate, startDelay);
        }
    }

}
